#*************************************************************
# Progamm:			isa_atmosphaere
# Sinn des Programms: 		Einlesen und Interpolieren sowie Errechnen von
#				athmosphärischen Zustandsgrößen aus einer Tabelle				
# Sinn der Einheit  :		Hauptprogramm	
# Name dieser Datei:		isa_atmosphaere_haupt.py
# Wird aufgerufen von:		-
# Ruft auf:			steuer.py, isa_lese.py, rechner.py, ausgabe.py
# Nummer im Programmpaket:	1
# von:				5			
# Autor: 			Armin Ulrich
# Erster Lauf: 			04.12.2020
# Letzte Änderung: 		31.07.2021
#*************************************************************


#*************************************************************
# Importieren der Klassen zur Zeitdokumentation
#*************************************************************
import time
import datetime
	
#*************************************************************
# Importieren der Klassen
#*************************************************************

from steuer import Steuer
from eingabe import Eingabe
from rechner import Rechner
from ausgabe import Ausgabe
#*************************************************************
# Belegung der Klassen
#*************************************************************
steuer = Steuer()
eingabe = Eingabe()
rechner = Rechner()
ausgabe = Ausgabe()

#*************************************************************
# Berechnung der Startzeit des Programmes mit:
# a) datetime
# b) time
# und Konvertieren der Startzeit zu einem String "datum_zeit"
#*************************************************************
dt_start = datetime.datetime.now()
lt_start = time.localtime()
jahr, monat, tag, stunde, minute, sekunde  = lt_start[0:6]
datum_zeit = "Datum: " +str(tag)+"."+str(monat)+"."+str(jahr)+"\n"+"Zeit: "+str(stunde)+":"+str(minute)+":"+str(sekunde)


#*************************************************************
# Ausgabe an
#*************************************************************
print("ISA-Atmosphäre")

  
#*************************************************************
# Öffnen der Logdatei
#*************************************************************
fobj_log = open("isa_atmosphaere.log","w")
#*************************************************************
# Schreiben von Datum und Zeit des Programmstarts 
# in die Logdatei 
#*************************************************************
print("Anfang: "+ "\n" + datum_zeit)
fobj_log.write("Anfang: "+ "\n" + datum_zeit)
fobj_log.write("\n")

if steuer.steuer_input():
  fobj_log.write( "Steuerdatei existiert\n")
  if eingabe.isa_input(steuer.eingabe_datei):
    fobj_log.write( "Datei existiert\n")
    rechner.gui()
    if rechner.berechnung(eingabe.max_zeile, eingabe.höhe,eingabe.T,eingabe.p,eingabe.rho_relativ,eingabe.nue,eingabe.lambada,eingabe.a):
      # Schema: interpoliert, formel, interpoliert, formel,.......
      ausgabe.schirm_ausgabe_einzel(rechner.h, rechner.T_interpoliert, rechner.T_formel,  rechner.p_interpoliert, rechner.p_formel,  rechner.rho_relativ_interpoliert, \
       rechner.rho_relativ_formel, rechner.nue_interpoliert, rechner.nue_formel,  rechner.a_interpoliert, rechner.a_formel)
      ausgabe.schirm_ausgabe_tabelle(eingabe.max_zeile,eingabe.höhe,eingabe.T ,rechner.T_formel_feld,eingabe.p ,rechner.p_formel_feld,eingabe.rho_relativ ,rechner.rho_relativ_formel_feld,\
       eingabe.nue ,rechner.nue_formel_feld, eingabe.a ,rechner.a_formel_feld)
      
      ausgabe.datei_ausgabe_einzel(steuer.ausgabe_datei_einzel, rechner.h, rechner.T_interpoliert, rechner.T_formel, rechner.p_interpoliert, rechner.p_formel, \
       rechner.rho_relativ_interpoliert, rechner.rho_relativ_formel,rechner.nue_interpoliert, rechner.nue_formel, rechner.a_interpoliert , rechner.a_formel)
      ausgabe.datei_ausgabe_tabelle(steuer.ausgabe_datei,eingabe.max_zeile,eingabe.höhe,eingabe.T ,rechner.T_formel_feld,eingabe.p ,rechner.p_formel_feld,\
      eingabe.rho_relativ ,rechner.rho_relativ_formel_feld, eingabe.nue ,rechner.nue_formel_feld, eingabe.a ,rechner.a_formel_feld )
      
      ausgabe.gui_ausgabe(rechner.h, rechner.T_interpoliert, rechner.T_formel,  rechner.p_interpoliert, rechner.p_formel,  rechner.rho_relativ_interpoliert, rechner.rho_relativ_formel,  \
      rechner.nue_interpoliert, rechner.nue_formel, rechner.a_interpoliert , rechner.a_formel)
      ausgabe.graphik_ausgabe(eingabe.max_zeile, eingabe.höhe,eingabe.T ,rechner.T_formel_feld ,True, 'temperatur', steuer.graphik_format, 'Temperatur', 'T [K]')
      ausgabe.graphik_ausgabe(eingabe.max_zeile, eingabe.höhe,eingabe.p ,rechner.p_formel_feld ,True, 'druck', steuer.graphik_format, 'Druck', 'p [pa]')
      ausgabe.graphik_ausgabe(eingabe.max_zeile, eingabe.höhe,eingabe.rho_relativ ,rechner.rho_relativ_formel_feld ,True, 'relative_dichte', steuer.graphik_format, 'Relative Dichte',\
      'rho_relativ [-]')
      ausgabe.graphik_ausgabe(eingabe.max_zeile, eingabe.höhe,eingabe.nue ,rechner.nue_formel_feld ,True, 'kinematische_zaehigkeit', steuer.graphik_format, 'Kinematische Zähigkeit', \
      'nue [m²/s]')
      ausgabe.graphik_ausgabe(eingabe.max_zeile, eingabe.höhe,eingabe.a ,rechner.a_formel_feld ,True, 'schallgeschwindigkeit', steuer.graphik_format, 'Schallgeschwindigkeit', 'a [m/s]')
    else:
      print("Fehler in Berechnung!")  
  else:
    fobj_log.write( "Datei existiert nicht\n")
    
else:
  fobj_log.write( "Steuerdatei existiert nicht\n")






#*************************************************************
# Berechnung der Endzeit des Programmes mit:
# a) datetime
# b) time
# und Konvertieren der Endzeit zu einem String "datum_zeit"
#*************************************************************
lt_end = time.localtime()
jahr, monat, tag, stunde, minute, sekunde  = lt_end[0:6]
datum_zeit = "Datum: "+str(tag)+"."+str(monat)+"."+str(jahr)+"\n"+"Zeit: "+str(stunde)+":"+str(minute)+":"+str(sekunde)
dt_end = datetime.datetime.now()

#*************************************************************
# Berechnung der Rechendauer mit dem Zeitdifferenzoperator 
# des Moduls datetime
#*************************************************************
dauer = dt_end - dt_start

#*************************************************************
# Schreiben von Datum und Zeit des Programmendes und der 
# Rechendauer in die Logdatei 
#*************************************************************
fobj_log.write("Ende: "+ "\n" +datum_zeit + "\n")
fobj_log.write("Rechendauer: "+ "\n" + str(dauer))
print("Ende: "+ "\n" +datum_zeit)
print("Rechendauer: "+ "\n" + str(dauer))
#*************************************************************
# Schließen der Logdatei
#*************************************************************
fobj_log.close()

#*************************************************************
#Programmende
#*************************************************************


