#*************************************************************
# Progamm:			isa_atmosphaere
# Sinn des Programms: 		Einlesen und Interpolieren sowie Errechnen von
#				athmosphärischen Zustandsgrößen aus einer Tabelle				
# Sinn der Einheit  :		Hauptprogramm	
# Name dieser Datei:		rechner.py
# Wird aufgerufen von:		isa_atmosphaere_haupt.py
# Ruft auf:			-
# Nummer im Programmpaket:	4
# von:				5			
# Autor: 			Armin Ulrich
# Erster Lauf: 			31.05.2021
# Letzte Änderung: 		31.07.2021
#*************************************************************
import math
import numpy as np

class Rechner:

  def __init__(self):		
    print("Rechnerobjekt erzeugt")


  #Destruktormethode
  def __del__(self):	
    print("Rechnerobjekt entfernt")   
    
  #Funktion zum Auflösen der Benutzeroberfläche
  def weiter(self):
    self.eingabe_gui.destroy()  
  
  def berechnung(self,max_zeile,höhe,T,p,rho_relativ,nue,lambada,a):
    try:
      self.formel()  
      self.tabelle(max_zeile,höhe,T,p,rho_relativ,nue,lambada,a)  
      self.vergleich(max_zeile,höhe,T,p,rho_relativ,nue,lambada,a)  
      return True
    except:
      return False
  
  def formel(self):
    
    print("Formel", self.h)
# Berechnung der Werte am Boden h = 0    
    self.T_0 = 288.0 # K
    self.p_0 = 1.013e5 # N/m²
    self.rho_0 = 1.225 # kg/m³
    self.rho_relativ = 1.0 # -
    self.L = 0.0065 # K/m
    self.g = 9.81 # m/s²
    self.R = 287.0 # J/(kg*K)
    self.kappa = 1.4 # -
    
    
# Berechnung der Werte bei der Tropopausenhöhe h = 11000
    self.h_Tropopause = 11000.0
    self.T_Tropopause = self.T_0 - self.L*self.h_Tropopause
    self.p_Tropopause =  self.p_0 * (1.0 - self.L*self.h_Tropopause/self.T_0)**(self.g/(self.R*self.L))
    self.rho_Tropopause = (1.0 - self.L*self.h_Tropopause/self.T_0)**((self.g/(self.R*self.L)) - 1.0)*self.rho_0  
    self.T_formel = self.T_formel_funktion(self.h)
    self.p_formel=self.p_formel_funktion(self.h)
    rho_formel = self.rho_formel_funktion(self.h)
    self.rho_relativ_formel = self.rho_relativ_formel_funktion(self.h)
    self.nue_formel = self.nue_formel_funktion(self.h)    
    self.a_formel = self.a_formel_funktion(self.h)
      
    
  def T_formel_funktion(self, h):
    if h <= self.h_Tropopause:
      T_formel = self.T_0 - self.L*h
    else: 
      T_formel = self.T_Tropopause  
    # print("T_formel =", T_formel)  
    return T_formel
  
  def p_formel_funktion(self, h): 
    if h <= self.h_Tropopause:
      p_formel = self.p_0 * (1.0 - self.L*h/self.T_0)**(self.g/(self.R*self.L)) 
    else: 
      p_formel = self.p_Tropopause * math.exp((self.h_Tropopause - h)*self.g/(self.R*(self.T_0 - self.L*self.h_Tropopause)) )
    # print("p_formel =", p_formel)
    return p_formel 
    
    
  def rho_formel_funktion(self, h): 
    if self.h <= self.h_Tropopause:
      rho_formel = self.rho_0 * (1.0 - self.L*h/self.T_0)**((self.g/(self.R*self.L))-1.0) 
    else: 
      rho_formel = self.rho_Tropopause * math.exp((self.h_Tropopause - h)*self.g/(self.R*(self.T_0 - self.L*self.h_Tropopause)) )
    # print("rho_formel =", rho_formel)
    return rho_formel
  
  def rho_relativ_formel_funktion(self,h):
    rho_relativ_formel = self.rho_formel_funktion(h)/self.rho_0
    return rho_relativ_formel
    
  def nue_formel_funktion(self,h):
    T = self.T_formel_funktion(self.h)
    nue_formel = 1.458e-6* (T**(3.0/2.0))/(T+110.4)/self.rho_formel_funktion(h) 
    return nue_formel
    
  def a_formel_funktion(self,h):
    a_formel = math.sqrt(self.kappa*self.R*self.T_formel_funktion(h))
    return a_formel  
    
  def start_rechnung(self):
    try:
      self.h = float(self.e.get())
      self.lb1["text"] = "Eingabe o.k. Bitte auf 'weiter' drücken!" 
    except:
      self.lb1["text"] = "Bitte Zahl eingeben! Dezimalpunt zulässig." 
    if self.h >= 30000 :
      self.lb1["text"] = "Bitte Höhe neu eingeben! \n 0 <= Höhe < 30000"   
      
      
  def tabelle(self,max_zeile,höhe,T,p,rho_rel,nue,lambada,a):
    self.T_interpoliert = self.interpolation(max_zeile,self.h,höhe,T) 
    self.p_interpoliert = self.interpolation(max_zeile,self.h,höhe,p)
    self.rho_relativ_interpoliert = self.interpolation(max_zeile,self.h,höhe,rho_rel)
    self.nue_interpoliert = self.interpolation(max_zeile,self.h,höhe,nue)
    self.lambada_interpoliert = self.interpolation(max_zeile,self.h,höhe,lambada)
    self.a_interpoliert = self.interpolation(max_zeile,self.h,höhe,a)
    
  def interpolation(self,max_zeile,h,höhe,größe):
    for i in range(max_zeile-1):  #self.zeilenzähler
      if ((höhe[i]<=h) & (h<höhe[i+1])):
        größe_interpoliert=größe[i]*(höhe[i+1]-h)/(höhe[i+1]-höhe[i])+größe[i+1]*(h-höhe[i])/(höhe[i+1]-höhe[i])
    # print(h,größe_interpoliert)
    return größe_interpoliert  
    
  def vergleich(self,max_zeile,höhe,T,p,rho_relativ,nue,lambada,a):
    self.T_formel_feld = np.zeros(max_zeile)
    self.p_formel_feld = np.zeros(max_zeile)
    self.rho_relativ_formel_feld = np.zeros(max_zeile)
    self.nue_formel_feld = np.zeros(max_zeile)
    self.a_formel_feld = np.zeros(max_zeile)
    
    # self.höhe_formel_feld = höhe
    
    for i in range(max_zeile):
      self.T_formel_feld[i] = self.T_formel_funktion(höhe[i])
      self.p_formel_feld[i] = self.p_formel_funktion(höhe[i])
      self.rho_relativ_formel_feld[i] = self.rho_relativ_formel_funktion(höhe[i])
      self.nue_formel_feld[i] = self.nue_formel_funktion(höhe[i])
      self.a_formel_feld[i] = self.a_formel_funktion(höhe[i])
      # print("Vergleich:" , höhe[i], T[i], self.T_formel_feld[i])
    
  def gui(self):
    import tkinter
    self.eingabe_gui = tkinter.Tk()
    
    # Eingabeaufforderung
    
    self.lb1 = tkinter.Label(self.eingabe_gui, text = "Bitte Höhe in Meter eingeben: 0 <= h < 30000:")
    self.lb1.pack()

    
    # einzeiliges Eingabefeld    
    self.e = tkinter.Entry(self.eingabe_gui)
    self.e.pack() 
    
    button_höhe = tkinter.Button(self.eingabe_gui, text = "Übernahme der eingegebenen Höhe", command = self.start_rechnung)
    button_höhe.pack()
    button_weiter = tkinter.Button(self.eingabe_gui, text = "weiter", command = self.weiter)
    button_weiter.pack()

    #Endlosschleife
    self.eingabe_gui.mainloop()
    
    
